eval 'exec perl -w -S $0 ${1+"$@"}'
  if 0;

# dvdgm.pl - simple DVD menu generator for use with spumux/dvdauthor
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.

use strict;
use GD;
use Config::IniFiles;
use Getopt::Long qw(:config no_getopt_compat
		    gnu_compat permute no_ignore_case);


my $version               = "V0.90";
my $help;
my $config_file           = "dvdgm.cfg";
my $heading               = "";
my $sub_heading           = "";
my @button_text;

my $top_margin            = 60;
my $heading_gap           = 50;
my $sub_heading_gap       = 30;
my $button_gap            = 20;
my $bottom_margin         = 50;
my $heading_font          = "./arial.ttf";
my $heading_font_size     = 40;
my $sub_heading_font      = "./arial.ttf";
my $sub_heading_font_size = 18;
my $button_font           = "./arial.ttf";
my $button_font_size      = 30;
my @heading_color         = (0xff, 0xff, 0xff);
my @sub_heading_color     = (0x80, 0x80, 0x80);
my @button_base_color     = (0x80, 0x80, 0x80);
my @button_hilite_color   = (0xff, 0xff, 0xff);
my @button_select_color   = (0xff, 0x00, 0x0);
my $tv_standard           = "PAL";
my $width                 = 720;
my $height                = 576;
my $menu_base             = "menu-base.png";
my $menu_hilite           = "menu-hilite.png";
my $menu_select           = "menu-select.png";
my $xml_header            = "";
my $xml_button            = "%d %d %d %d\n";
my $xml_footer            = "";


sub usage
{
  print STDERR "dvdgm $version\n";
  print STDERR "\n";
  print STDERR "Usage: dvdgm.pl [--config|-c config-file]\n";
  print STDERR "                [--heading|-H heading]\n";
  print STDERR "                [--subheading|-s subheading]\n";
  print STDERR "                [--help|-h]\n";
  print STDERR "                menu-item ...\n";
  print STDERR "\n";
  print STDERR "config-file defaults to <dvdgm.cfg>\n";

  exit (1);
}


sub read_config
{
  my $cfg = Config::IniFiles -> new (-file => $config_file);

  unless ($cfg)
  {
    foreach my $s (@Config::IniFiles::errors)
    {
      print STDERR "$config_file: $s\n";
    }

    usage ();
  }

  $top_margin            = $cfg -> val ("General", "top_margin");
  $bottom_margin         = $cfg -> val ("General", "bottom_margin");
  $tv_standard           = $cfg -> val ("General", "tv_standard");
  $height                = 480 if ($tv_standard ne "PAL");

  @heading_color         = split (/,\s*/, $cfg -> val ("Heading", "color"));
  $heading_font          = $cfg -> val ("Heading", "font");
  $heading_font_size     = $cfg -> val ("Heading", "font_size");
  $heading_gap           = $cfg -> val ("Heading", "gap");

  @sub_heading_color     = split (/,\s*/, $cfg -> val ("SubHeading", "color"));
  $sub_heading_font      = $cfg -> val ("SubHeading", "font");
  $sub_heading_font_size = $cfg -> val ("SubHeading", "font_size");
  $sub_heading_gap       = $cfg -> val ("SubHeading", "gap");

  @button_base_color     = split (/,\s*/,
				  $cfg -> val ("Buttons", "base_color"));
  @button_hilite_color   = split (/,\s*/,
				  $cfg -> val ("Buttons", "hilite_color"));
  @button_select_color   = split (/,\s*/,
				  $cfg -> val ("Buttons", "select_color"));
  $button_font           = $cfg -> val ("Buttons", "font");
  $button_font_size      = $cfg -> val ("Buttons", "font_size");
  $button_gap            = $cfg -> val ("Buttons", "gap");

  $menu_base             = $cfg -> val ("Files", "menu_base");
  $menu_hilite           = $cfg -> val ("Files", "menu_hilite");
  $menu_select           = $cfg -> val ("Files", "menu_select");

  $xml_header            = $cfg -> val ("XML", "header");
  $xml_button            = $cfg -> val ("XML", "button");
  $xml_footer            = $cfg -> val ("XML", "footer");
}


# figure out width and height of text drawn with color, font and size
sub text_bounds
{
  my $color = shift;
  my $font  = shift;
  my $size  = shift;
  my $text  = shift;

  my @bounds = GD::Image -> stringFT ($color, $font, $size, 0, 0, 0, $text);

  my $w = $bounds [4] - $bounds [0];
  my $h = $bounds [1] - $bounds [5];

  return ($w, $h);
}


# find the maximum height of texts drawn with color, font and size
sub max_text_height
{
  my $color = shift;
  my $font  = shift;
  my $size  = shift;
  my @text  = @{shift ()};

  my $rv = 0;

  for my $t (@text)
  {
    my ($w, $h) = text_bounds ($color, $font, $size, $t);

    $rv = $h if ($h > $rv);
  }

  return $rv;
}


# main
read_config ();
usage () if (!GetOptions ("config=s"     => \$config_file,
			  "heading|H=s"  => \$heading,
			  "subheading=s" => \$sub_heading,
			  "help|h"       => \$help) ||
	     $help || !@ARGV);

@button_text = @ARGV if @ARGV;

# GD renders TrueType fonts better in a truecolor image
# we turn them back to indexed later
GD::Image -> trueColor (1);

# create base image and allocate colors
my $b_img   = GD::Image -> new ($width, $height);
my $b_black = $b_img -> colorAllocate (0, 0, 0);
my $b_hcol  = $b_img -> colorAllocate (@heading_color);
my $b_scol  = $b_img -> colorAllocate (@sub_heading_color);
my $b_bcol  = $b_img -> colorAllocate (@button_base_color);

# create highlight image and allocate colors
my $h_img   = GD::Image -> new ($width, $height);
my $h_black = $h_img -> colorAllocate (0, 0, 0);
my $h_bcol  = $h_img -> colorAllocate (@button_hilite_color);

# create select image and allocate colors
my $s_img   = GD::Image -> new ($width, $height);
my $s_black = $s_img -> colorAllocate (0, 0, 0);
my $s_bcol  = $s_img -> colorAllocate (@button_select_color);

# turn black into transparent
$b_img -> transparent ($b_black);
$h_img -> transparent ($h_black);
$s_img -> transparent ($s_black);

my ($x, $y);

# draw heading on base image
my ($w, $h) = text_bounds ($b_hcol, $heading_font,
			   $heading_font_size, $heading);

$y = $top_margin + $h;
$x = ($width - $w) / 2;

$b_img -> stringFT (-$b_hcol, $heading_font,
		    $heading_font_size, 0, $x, $y, $heading);

# draw sub-heading on base image
($w, $h) = text_bounds ($b_scol, $sub_heading_font,
			$sub_heading_font_size, $sub_heading);

$y += $heading_gap + $h;
$x  = ($width - $w) / 2;

$b_img -> stringFT (-$b_scol, $sub_heading_font,
		    $sub_heading_font_size, 0, $x, $y, $sub_heading);

$y += $sub_heading_gap;

# calculate maximum button height
my $button_height =
  max_text_height ($b_bcol, $button_font, $button_font_size, \@button_text);

# reduce button font size if not all selections fit on screen
while (@button_text * ($button_height + $button_gap) >=
       $height - $bottom_margin - $y)
{
  $button_font_size -= 0.5;

  $button_height =
    max_text_height ($b_bcol, $button_font, $button_font_size, \@button_text);
}

print "$xml_header\n";

# draw the menu buttons
for my $text (@button_text)
{
  ($w, $h) = text_bounds ($b_bcol, $button_font, $button_font_size, $text);

  my $x0 = ($width - $w) / 2;
  my $x1 = $x0 + $w + 1;
  my $y0 = $y;
  my $y1 = $y + $button_height;

  # print button corner coordinates appropriate for use with spumux
  printf "$xml_button\n", $x0, $y0, $x1+1, $y1+1;

  $x = $x0;
  $y = $y1;

  # draw text on base image
  $b_img -> stringFT (-$b_bcol, $button_font,
		      $button_font_size, 0, $x, $y, $text);

  # draw text on highlight image
  $h_img -> stringFT (-$h_bcol, $button_font,
		      $button_font_size, 0, $x, $y, $text);

  # draw text on select image
  $s_img -> stringFT (-$s_bcol, $button_font,
		      $button_font_size, 0, $x, $y, $text);

  $y += $button_gap;
}

print "$xml_footer\n";

# even if we do not want dithering, we enable it here
# since we get ugly color interpolation otherwise
$b_img -> trueColorToPalette (1, 4);
$h_img -> trueColorToPalette (1, 2);
$s_img -> trueColorToPalette (1, 2);

open (IMG, ">", $menu_base) || die "$menu_base: $!\n";
binmode (IMG);
print IMG $b_img -> png (0);
close (IMG);

open (IMG, ">", $menu_hilite) || die "$menu_hilite: $!\n";
binmode (IMG);
print IMG $h_img -> png (0);
close (IMG);

open (IMG, ">", $menu_select) || die "$menu_select: $!\n";
binmode (IMG);
print IMG $s_img -> png (0);
close (IMG);
